      subroutine bgf2
     & (sntr, zcctr, cmean, crrmn, dbhmn, 
     & ggsu, ggvu, utot, glmp, zz1, zz2, zz3, sths, cths, 
     & sthv, cthv, calph, poouj, poodj)
c
c Calculation of bidirectional free lines of sight outside the tree crown,
c one ray exits the crown at level z1, the other ray at level z2, the angle 
c between rays is alph.  A. Kuusk 29.01.2024
c   
c  Input:
c  Output:
c    poouj, poodj
c   pooui - upper hemisphere, poodj - lower hemisphere
c
      include 'par.h'
c
*     data eps1/.1d-3/, eps2/.2d-5/, eps3/.1d-6/
      data eps/.1d-4/
c
c
      aell = crrmn
      cell = cmean
      hbase = max((zcctr - cmean), 0.d0)
      htr = zcctr + cmean
      alph = acos(calph)
c                                              sun direction
      ugx = ggsu*utot
      sthx = sths
      cthx = cths
      zzx  = zz1
      if (zzx .ge. htr) then
        szs = 0.d0
        tauzs = 0.d0
      else
        call gfzx
     & (sntr, zcctr, cell, aell, dbhmn, ugx, 
     &   glmp, zzx, sthx, cthx, sproj, tauz)
        tauzs = tauz
        szs = sproj
      endif
c                                            view direction up
      ugx = ggvu*utot
      sthx = sthv
      cthx = cthv
      zzx  = zz2
      if (zzx .ge. htr) then
        szv = 0.d0
        tauzv = 0.d0
      else
        call gfzx
     & (sntr, zcctr, cell, aell, dbhmn, ugx, 
     &   glmp, zzx, sthx, cthx, sproj, tauz)
        tauzv = tauz
        szv = sproj
      endif
c                crown level hot-spot correction for the bidirectional 
c                free lines of sight outside the tree crown 
      if ((1.d0 - calph) .lt. eps) then
        tausv = tauzs
      else
        if (szv .gt. szs) then
          tauc = tauzs*(calph/(1.d0 + alph))**4
        else
          tauc = tauzv*(calph/(1.d0 + alph))**4
        endif
        tausv = tauzs + tauzv - tauc
      endif
      poouj = exp(-tausv) 
c                                                view direction down
c                           crown projection in view direction (down)
      if (zz3 .gt. hbase) then
        tgthv = sthv/cthv
        call pelld
     &     (tgthv, zz3, htr, cell, aell, szd3, vzdj)
c
        if (szd3 .gt. 0.d0) then
          rllv2d = vzdj/szd3/cthv
*         rllv2d = vzdj/szd3/cthv*clratio
          azvd  = exp(-ggvu*rllv2d)
        else
          azvd  = 0.d0
*         azvd  = 1.d0
        endif
      else
        azvd  = 0.d0
	szd3  = 0.d0
      endif
c                            stem projection in view direction (down)
      slty3 = min(zz3, hbase)
      zx1 = 0.d0
      zx2 = slty3
      call stem(zx1, zx2, dbhmn, htr, styx)
      sty3  = styx*tgthv
c                            pooj down using binome equation, 28.05.1998
c                            the first coefficient
      b11j = bij(azvd, glmp)
      sumd = sntr*(b11j*szd3 + sty3)
c
*        dr = 0.0174532925d0
*        write (*, '(f6.1, 6f10.5)') 
*    &   asin(sthv)/dr, b11j, azvd, rllv2d, vzdj, szd3, sty3
c
      poodj = exp(-sumd)
c
      return
      end
*
************************************************************************
*
