      function func
     & (nxx, xxf)
c
c  The interface between the main program FRT and subroutines for the
c  calculation of forest reflectance and transmittance, and the merit
c  function for the inversion of the FRT.  A. Kuusk, 24.10.1995 - Nov. 2009
c  In the iversion the reflectance and/or transmittance at several
c  wavelengths and view directions can be used.
c
c     Elliptical LAD for tree foliage   12.2012
c
c     The allometric regressions for tree parameters in the 
c     subroutine regre, Feb. 1998, Dec. 2000
c
c     Fisher's grouping index, March 1998
c
c     Ground albedo is calculated integrating over polar angle 
c     in the principal plane phi = pi/2, A. Kuusk  7.10.1998
c  
c     The expert guess in the merit function, May 2001.
c
c     The 1997 version of PROSPECT, May 2001.
c
c     Two-layer homogeneous model ACRM for the ground vegetation,
c     and two leaf optics models (PROSPECT and LIBERTY), Feb. 2002.
c
c    Exchange of functions and parameters between frt.f and func using 
c    arrays ?_* in common areas
c
      include 'par.h'
      include 'stands.h'
      include 'spectra.h'
      include 'func.h'
c
      save f, ncl, jx, nbnds
      save x0, xx, xa, xb, xdelt, llxx
c
      dimension xx(nvar), xxf(nvar), xa(nvar), xb(nvar), xdelt(nvar), 
     & llxx(nvar), x0(nvar)
c
      common /par/  zeps, tol, tiny, ftol, alphopt, betopt, gammopt, 
     & at, dxopt, ier, ncount, nfmax, itmax, itbr, nbrak, ilo
c
      data lstruc/.true./, lopt/.true./
c
      ncount = ncount + 1
      f   = 0.d0
c
      ijob   = i_job 
c                                                 ! The first pass
      if (ncount .eq. 1) then
        ncl     = n_cl
        nbnds   = n_bnds
        clmpst  = x_c(1, 13)
        call iterats(clmpst, glmp)
c
        if (ncl .gt. 1) then
          do jcl = 2, ncl
            stdns(jcl)  = x_c(jcl, 1)
            htr(jcl)    = x_c(jcl, 2)
            hc1(jcl)    = x_c(jcl, 3)
            hc2(jcl)    = x_c(jcl, 4)
            rcr(jcl)    = x_c(jcl, 5)
            dbh(jcl)    = x_c(jcl, 6)*1.d-2
            rmass(jcl)  = x_c(jcl, 7)
            slwcl(jcl)  = x_c(jcl, 8)
            eln3(jcl)   = x_c(jcl, 9)
            thm3(jcl)   = x_c(jcl, 10)*dr
            shl(jcl)    = x_c(jcl, 11)
            if (shl(jcl) .le. 0.d0) shl(jcl) = 0.01d0
            rlaicl(jcl) = stdns(jcl)*rmass(jcl)*1.d3/slwcl(jcl)
            rbai(jcl)   = x_c(jcl, 12)
            x_c(jcl, 13) = clmpst
            baicl(jcl)  = rlaicl(jcl)*rbai(jcl)
            ghg(jcl)    = x_c(jcl, 14)
            cshj        = x_c(jcl, 15)
            clmpsh(jcl) = cshj
            rnlfc(jcl)  = x_c(jcl, 16)
            vaicl(jcl)  = x_c(jcl, 17)/cshj
            dcl(jcl)    = x_c(jcl, 18)
            acl(jcl)    = x_c(jcl, 19)
            do jc = 1, ncomp3(jcl)
              ccomp3(jcl, jc) = x_c(jcl, jc+19)
            enddo
          enddo
        endif
c
        if (ijob .gt. 2) then
          do jx = 1, nvar
            llxx(jx)  = l_lxx(jx)
            x0(jx)    = x_0(jx)
            xx(jx)    = x_x(jx)
            xa(jx)    = x_a(jx)
            xb(jx)    = x_b(jx)
            xdelt(jx) = x_delt(jx)
          enddo
        endif
c
        do jx = 1, nvar
          xx(jx)    = x_x(jx)
        enddo
c                                                   ! endif(ncount=1)
      endif
c                                ! check constraints and scale variables
      if (ijob .gt. 2) then
        do i = 1, nxx
          li  = llxx(i)
          xi3 = xxf(i)
          if (li .eq. 13) lstruc = .true.
          x2  = 0.d0
          if (xi3 .lt. -5.d0) then
             x2  = xi3 + 5.d0
             xi3 =  -5.d0
          else if (xi3 .gt. 5.d0) then
             x2  = xi3 - 5.d0
             xi3 = 5.d0
          endif
          xli    = xi3/xb(li) + xa(li)
          xx(li) = xli
          xi4    = x2*x2*at
          f      = f + xi4 + ((xli - x0(li))/xdelt(li))**2
        enddo
      endif
c                                   ! initialisation of model parameters
      stdns1   = xx(1)
      stdns(1) = stdns1
      htr(1)   = xx(2)
      hc1(1)   = xx(3)
c
      if (hc1(1) .gt. htr(1)) then
        hc1(1) = htr(1)
        print *, 'FUNCIFRT:'
        print *, 'Warning: tree height is less than crown length,'
        print *, 'the crown length is adjusted.'
      endif
c
      hc2(1)   = xx(4)
      rcr(1)   = xx(5)
      dbh(1)   = xx(6)*1.d-2
      rmass(1) = xx(7)
      slwcl(1) = xx(8)
      eln3(1)  = xx(9)
      thm3(1)  = xx(10)*dr
      shl(1)   = xx(11)
      if (shl(1) .le. 0.d0) shl(1) = .01d0
c
      rlaicl(1) = stdns(1)*rmass(1)*1.d3/slwcl(1)
      baicl(1) = xx(12)*rlaicl(1)
c
*     if (lhtr .or. lhc1 .or. lrcrwn .or. lmass .or. lbai) then
*       call regre
*    &     (lmass, lbai, lhtr, lrcrwn, lhc1,
*    &     xx, rlai, htr, hc1, rcr)
*     endif
c
      clmpx  = xx(13)
      clmpst = clmpx
      call iterats(clmpx, fgx)
      glmp      = fgx
      ghg(1)    = xx(14)
      clmpsh(1) = xx(15)
      rnlfc(1)  = xx(16)
      vaicl(1)  = xx(17)/xx(15)
      dcl(1)    = xx(18)
      acl(1)    = xx(19)
c
      do jc = 1, ncomp3(1)
        ccomp3(1, jc) = xx(jc+19)
      enddo
c                        ground vegetation, upper layer
      ul2   = xx(30)
      sl2   = xx(31)
      clmp2 = xx(32)
      szz   = xx(33)
      eln2  = xx(34)
      thm2  = xx(35)*dr
      rnc2  = xx(36)
      slw2  = xx(37)
      vai2  = xx(38)/clmp2
      do jc = 1, ncomp2
        ccomp2(jc) = xx(jc+38)
      enddo
c                        ground vegetation, lower layer
      ul1   = xx(49)
      sl1   = xx(50)
      clmp1 = xx(51)
      eln1  = xx(52)
      thm1  = xx(53)*dr
      rnc1  = xx(54)
      slw1  = xx(55)
      vai1  = xx(56)/clmp1
      do jc = 1, ncomp1
         ccomp1(jc) = xx(jc+56)
      enddo
c                                        soil
      rsl1  = xx(67)
      rsl2  = xx(68)
      rsl3  = xx(69)
      rsl4  = xx(70)
c
      lstruc  = .true.
      lopt    = .true.
      if (ijob .eq. 0) then
        jx = 1
        call frtsv(ncl, jx)
      endif
c                                        ! Loop for wavelength
      if (ijob .eq. 1) then
        do jbnd = 1, nbnds
          lopt    = .true.
          jx = jbnd
          call frtsv(ncl, jx)
          jpass = jpass + 1
          lstruc = .false.
        enddo
      endif
c                                        ! Loop for view angles
      if (ijob .eq. 2) then
        jbnd = 1
        nbnds = 1
        jx = jview
        call frtsv(ncl, jx)
        jpass = jpass + 1
      endif
c
c
c                                   ! calculation of the merit function
      if (ijob .gt. 2) then
        lopt = .true.
        lstruc = .true.
        do jbnd = 1, nbnds
          jx = jbnd
          call frtsv(ncl, jx)
          bqq = rth(jx)
          xi4  = bqjarr(jx) - bqq
*       print *, 'HET48o: bqi, p2bq: ', bqi, p2bq
          if (ijob .eq. 3) then
            f    = f + xi4**2
          else
            f    = f + (xi4/drhoarr(jbnd))**2
          endif
          r_out(jbnd) = bqq
        enddo
      endif
      func  = f
*           print *, 'f  = ', f
      return
      end
*
************************************************************************
*
