      subroutine optmean
     & (ncl, jwl, tlty, tbai, tlaief, rleff, tleff, rty, 
     & rteff, tteff, rneff)
c
c  the mean values of optical parameters    A. Kuusk 23.09.1995
c
c  Input:
c    & (ncl, rlambda, tlty, tlai
c    
c  Output:
c    tleff, rleff, rneff, rty, rleff, tleff
c
c  tlai  - total LAI (leaves + branches + stems)
c  rleff - total efffective leaf reflectance
c  tleff - total efffective leaf transmittance
c
      include 'par.h'
      include 'stands.h'
      include 'spectra.h'
      dimension acoef1(nspchnl, ncomp0), acoef2(nspchnl, ncomp0)
      common /spect3/ acoef1, acoef2
c
      rind = refr(jwl)
c                                 ! Leaf reflectance and transmittance
      do jcl = 1, ncl
        dkl = 0.d0
        do jc = 1, ncomp3(jcl)
          dkl = dkl + ccomp3(jcl, jc)*acoef3(jcl, jwl, jc)
        enddo
        slw = slwcl(jcl)
        vai = vaicl(jcl)
        if (lmod3(jcl) .eq. 'prospect') then
          dkl = dkl/vai*slw*1.d-6
          call prospect(rind, dkl, vai, refl, tran)
        else if (lmod3(jcl) .eq. 'liberty') then
          rinf = -99.d0
          dcell0 = 80.d0
          asp = acl(jcl)
          coeff = dkl*dcl(jcl)/dcell0*slw*1.d-6/vai
          call liberty
     & (rind, coeff, vai, asp, rinf, refl, tran)
          if (refl .eq. rinf .or. rinf .lt. 0d0) then
            write (0, '(/a/a, f6.1, f10.4 /)') 'LIBERTY:',
     &        'Cannot calculate leaf reflectance, lambda, r_leaf = ', 
     &        rlmbd, refl
            stop
          endif
        else
          print *, 'ERR: No leaf optics model'
          stop
        endif 
c     
        rnlfj      = rnlfc(jcl)*rind
        if (rnlfj .lt. 1.d0) rnlfj = 1.d0
        rnlf(jcl)   = rnlfj
        rspj = ((1.d0 - rnlfj)/(1.d0 + rnlfj))**2
        rlfcl(jcl) = refl - rspj
        tlfcl(jcl) = tran
        rbrnc(jcl) = brrarr(jcl, jwl)
        rtrnk(jcl) = trrarr(jcl, jwl)
      enddo 
c
      sum2   = 0.d0
      do j = 1, ncl
        stdj    = stdns(j)
        htrj    = htr(j)
        hcrj    = hc1(j) + hc2(j)
        cli     = stdj*dbh(j)*.5d0*(htrj - hcrj)
        sum2    = sum2 + cli*rtrnk(j)
      enddo
      rty    = 0.d0
      if (tlty .ne. 0.d0) rty = sum2*pi/tlty
c
      sumn   = 0.d0
      sumlai = 0.d0
      sul0   = 0.d0
      sut    = 0.d0
      rteff  = 0.d0
      tteff  = 0.d0
c
      do jcl = 1, ncl
        rlaief       = rlaicl(jcl)*clmpsh(jcl)
        sumlai   = sumlai + rlaief
        baiclj       = baicl(jcl)
        claief       = rlaief + baiclj
        rdlf         = (rlaief*rlfcl(jcl) + baiclj*rbrnc(jcl))
        rrs(jcl, jwl) = rdlf/claief
        rnlfj        = rnlf(jcl)
c
        rteff        = rteff + (rlfcl(jcl) +
     &     ((1.d0 - rnlf(jcl))/(1.d0 + rnlf(jcl)))**2)*rlaief +
     &     rbrnc(jcl)*baiclj
        ttl          = rlaief*tlfcl(jcl)
        tteff        = tteff + ttl
        ttt(jcl, jwl) = ttl/claief
c
        sul0    = sul0 + rdlf
        sut     = sut + ttl
        sumn     = sumn + rlaief*rnlfj
      enddo
c
      suml  = tlaief + tbai
      utot  = suml + tlty
      rteff = (rteff + rty*tlty)/utot
      tteff = tteff/utot
      rneff = 1.d0
      if (sumlai .ne. 0.d0) rneff = sumn/sumlai
      tleff = sut/suml
      r12   = sul0/suml
      rleff = r12 + ((rneff - 1.d0)/(rneff + 1.d0))**2
c
      return
      end
*
************************************************************************
*
