      subroutine strmean
     & (ncl, ths, thv, sntr, htrmn, htrmax, crhmn, cilhmn, 
     & zcctr, cmean, crrmn, dbhmn, shlm, cshm, rirm, dcmn, acmn, 
     & crcl, cacl, tmass, slwm, efflai, tlty, tlaief, tlai, tbai, 
     & utot, uuz, thmm, ggsu, ggvu, ggmean, tsst, tsvt)
c
c  the mean values of structure parameters   
c  A. Kuusk  23.09.1995, 9.01.2024, 4.11.2024
c
c  Input:
c     ncl, ths, thv. 
c     Other input parameters are in the common /stndvars/,
c     stands.h, read in in the subroutine stands.
c
c  Output:
c    & sntr, htrmn, htrmax, crhmn, cilhmn, 
c    & zcctr, cmean, crrmn, dbhmn, shlm, dcmn, acmn,
c    & crcl, cacl, tlty, tlai, tbai, uuz, 
c    & ggsu, ggvu, ggeu, tsst, tsvt, tmass)
c
      include 'par.h'
      include 'stands.h'
c      
      dimension uuz(nzmax), rlaiz(nzmax)
c
*     data gsf/.5d0/, thseff/.698d0/
      data thseff/.698d0/
c       
      dz = 0.1d0
      cths = cos(ths)
      cthv = cos(thv)
      sthe = sin(thseff)
      cthe = cos(thseff)
      tgthe = sthe/cthe
c                                             ! total values
      sntr   = 0.d0                           ! #_of_trees
      htrmn  = 0.d0                           ! h_tree
      crhmn  = 0.d0                           ! crown length
      cilhmn = 0.d0
      crrmn  = 0.d0                           ! crown radius
      dbhmn  = 0.d0                           ! DBH
      tmass  = 0.d0
      slwm   = 0.d0
      thmm   = 0.d0
      shlm   = 0.d0
      rirm   = 0.d0
      tlaief = 0.d0
      tlai   = 0.d0
      tbai   = 0.d0
      tlty   = 0.d0
      tclai  = 0.d0
      cshm   = 0.d0
      do jz = 1, nzmax
         uuz(jz) = 0.d0
      enddo
c
      htrmax = 0.d0
      hcbmin = 50.d0
c                                             ! LIBERTY parameters
      dcmn   = 0.d0
      acmn   = 0.d0
      scrtot = 0.d0
      scacl  = 0.d0
      ggsu   = 0.d0
      ggvu   = 0.d0
      ggeu   = 0.d0
c                                            ! total values, mean values
      clmpj  = clmpst
      do jcl = 1, ncl
        stdj  = stdns(jcl)
        sntr  = sntr + stdj
        rcrj  = rcr(jcl)
        scrj  = pi*rcrj**2
        htrj  = htr(jcl)
        htrmn  = htrmn + stdj*htrj
        if (htrj .gt. htrmax) htrmax = htrj
        hc1j  = hc1(jcl)
        hc2j  = hc2(jcl)
        crhj  = hc1j + hc2j
        hcbj  = htrj - crhj
        if (hcbj .lt. hcbmin) hcbmin = hcbj
        dbhj  = dbh(jcl)
        rmj   = rmass(jcl)
        tmass = tmass + stdj*rmj
        slwj  = slwcl(jcl)
        slwm  = slwm + stdj*slwj
        cshj  = clmpsh(jcl)
        rirj  = rnlfc(jcl)
        rltyj = dbhj*(htrj - crhj)*pi/2.0d0*stdj
        tycl(jcl) = rltyj
        tlty  = tlty + rltyj
c
        trlaj  = rmj*1.d3/slwj
        tclaij = trlaj*cshj + baicl(jcl)
        rlaief = rlaicl(jcl)*cshj
        claief = rlaief + baicl(jcl)
        trbaj  = rbai(jcl)*trlaj
        uux    = (trlaj + trbaj)*cshj
        tlaij  = uux*stdj
        tlaij  = (trlaj*cshj + trbaj)*stdj
        tclai  = tclai + claief
        tlaief = tlaief + rlaief
        tlai   = tlai + rlaicl(jcl)
        tbai   = tbai + baicl(jcl)
c                                           ! crown volume
        if (lelli(jcl)) then
          volj = (2.d0/3.d0)*scrj*hc1j
        else
          volj = scrj*(hc1j/3.d0 + hc2j)
        endif
        uuu(jcl) = tclaij/volj
c   uuu(jcl) is the foliage area density in the crown of tree_jcl
*       tvol   = tvol + stdj*volj
c                                           ! leaf area & leaf area density
        crhmn  = crhmn + stdj*crhj
        cilhmn = cilhmn + stdj*hc2j
        crrmn  = crrmn + stdj*rcrj
        dbhmn  = dbhmn + stdj*dbhj
        shlm   = shlm + rlaief*shl(jcl)
        rirm   = rirm + rlaief*rirj
c                                           ! cacl & crcl
c                       ! the regularity parameter of the 1st tree class 
c                       ! is used for all classes
        crclj  = stdj*scrj
        scrtot = scrtot + crclj
        xxy = stdj*clmpj*scrj
        caclj  = 1.d0 - exp(-xxy)
        if (caclj .gt. crclj) then
          wvliit = crclj
          wcano  = caclj
          jwcl = jcl
        endif
        scacl  = scacl + clmpj*crclj
c                                           ! leaf area projection
        thmj  = thm3(jcl)
        elnj  = eln3(jcl)
        eej  = 1.d0 - exp(-elnj)
        ellmyj = abs(.059d0*eln3(jcl) *(thmj - 1.02d0) + .02d0)
        ellsij = abs(.0107d0 - .0216d0*eln3(jcl)*(thmj - .265d0))
        call ggg1(ths, thmj, eej, ellmyj, ellsij, ggs)
        gg3sj = ggs
        gg3s(jcl) = gg3sj
        call ggg1(thv, thmj, eej, ellmyj, ellsij, ggv)
        gg3vj = ggv
        gg3v(jcl) = gg3vj
        call ggg1(thseff, thmj, eej, ellmyj, ellsij, gge)
        gg3ej = gge
        ggsu = ggsu + tlaij*gg3sj
        ggvu = ggvu + tlaij*gg3vj
        ggeu = ggeu + tlaij*gg3ej
        thmm = thmm + rlaief*thmj
        cshm = cshm + rlaief*cshj
      enddo
c
      htrmn  = htrmn/sntr
      crhmn  = crhmn/sntr
      cilhmn = cilhmn/sntr
      crrmn  = crrmn/sntr
      dbhmn  = dbhmn/sntr
      tmass  = tmass/sntr
      slwm   = slwm/sntr
      thmm   = thmm/tlaief
      shlm   = shlm/tlaief
      cshm   = cshm/tlaief
      rirm   = rirm/tlaief
      crcl   = scrtot
      cacl   = 1.d0 - exp(-scacl)
*     dcmn   = dcmn/tlai
*     acmn   = acmn/tlai
c
************** Vertical profile of leaf area (18.12.2012) ****************
c              step = dz m
      jzmax = int(htrmax/dz)
      jzmin = max(int(hcbmin/dz)-1, 1)
      do jz = nzmax, jzmax+1, -1
        rlaiz(jz) = 0.d0
      enddo 
c
      sumlai = 0.d0
      do jz = jzmax, jzmin, -1
        zj = jz*dz
        do jcl = 1, ncl
          stdj   = stdns(jcl)
          rcrj   = rcr(jcl)
          htrj   = htr(jcl)
          hc1j   = hc1(jcl)
          hc2j   = hc2(jcl)
          if (lelli(jcl)) then
            zmin   = htrj - hc1j
          else 
            zmin   = htrj - hc1j - hc2j
          endif
          if (zj .gt. zmin .and. zj .lt. htrj) then
            if (lelli(jcl)) then
               cellj  = hc1j/2.d0
               z2 = zj - (zmin + htrj)/2.d0
               rz = rcrj*sqrt(cellj*cellj - z2*z2)/cellj
            else
               if (zj .lt. htrj-hc1j) then
                  rz = rcrj
               else
                  rz = rcrj*(htrj - zj)/hc1j
               endif
            endif
            ssz = pi*rz*rz
            udz = ssz*uuu(jcl)*stdj*dz
            uuz(jz) = uuz(jz) + udz
          endif
        enddo
        sumlai = sumlai + uuz(jz)
        rlaiz(jz) = sumlai
      enddo
      do jz = 1, jzmin-1
        rlaiz(jz) = rlaiz(jzmin)
      enddo
c
*     plevel = 0.0d0
      plevel = 0.05d0
*     plevel = 0.01d0
      p1 = plevel
      p2 = 1.d0 - plevel
      do jz = 1, jzmax
        if (rlaiz(jz)/rlaiz(1) .ge. p2) zmn1 = jz*dz
        if (rlaiz(jz)/rlaiz(1) .ge. p1) zmn2 = jz*dz
      enddo
      zcctr = (zmn1 + zmn2)/2.d0
      cmean = (zmn2 - zmn1)/2.d0
*     cmean = crhmn
      ggsu  = ggsu/tlai
      ggvu  = ggvu/tlai
      ggeu  = ggeu/tlai
      vzx = 4.d0/3.d0*pi*cmean*crrmn**2
*     utot  = tlai/(vzx*sntr)
      utot  = tlai/(vzx*sntr)/(1.d0 - plevel)
      tsst  = exp(-ggsu*clmpj*tlai/cths)
      tsvt  = exp(-ggvu*clmpj*tlai/cthv)
*     write (*, '(a, 2f6.1)') 'zmn1, zmn2: ', zmn1, zmn2
c
c                               LAI_eff for diffuse fluxes (09.2002) 
c                               thseff = 40° = .698 rad
      tgthx = tgthe
      zzx = 0.d0
      aell = crrmn
      cell = cmean
*     uge  = ggeu*tlai
      uge  = ggeu*tlai/(1.d0 - plevel)
      call pcrown
     &  (tgthx, zzx, zcctr, cell, aell, uge, 
     &   vzx, sproj, azxu)
*     rmnxy = vzx/sproj/cthe
*     azxu =   exp(-uge*rmnxy)
      a1gr = exp(-uge/(sntr*sproj*cthe))
      b10i = bij(a1gr, glmp)
      alsc = sntr*sproj*b10i
      efflai = cthe*alsc
      ggmean = ggeu
      return
      end
*
************************************************************************
*
